;; mh-folders.el
;; $Header$
;;;
;;;  Copyright (C) 1990 Free Software Foundation, Inc.
;;;
;;;     Author:  Alan Stebbens <aks@hub.ucsb.edu>
;;;	Please send suggestions and corrections to the above address.
;;;
;;; This file contains functions to enhance usage of recursive folders
;;; with the mh-e package, a GNU Emacs front end to the MH mail system.

;;; GNU Emacs is distributed in the hope that it will be useful, but
;;; without any warranty.  No author or distributor accepts
;;; responsibility to anyone for the consequences of using it or for
;;; whether it serves any particular purpose or works at all, unless he
;;; says so in writing.

;;; Everyone is granted permission to copy, modify and redistribute GNU
;;; Emacs, but only under the conditions described in the document "GNU
;;; Emacs copying permission notice".  An exact copy of the document is
;;; supposed to have been given to you along with GNU Emacs so that you
;;; can know how you may redistribute it all.  It should be in a file
;;; named COPYING.  Among other things, the copyright notice and this
;;; notice must be preserved on all copies.

;; Last Edited:
;; 
;; Mon Jun  4 10:36:14 1990 by Alan Stebbens (aks at somewhere.ucsb.edu)
;; 	 Was using mh-quit-kill-buffers wrong in mh-quit; corrected logic.
;; 
;; Tue May 29 14:58:34 1990 by Alan Stebbens (aks at somewhere.ucsb.edu)
;; 	 Added mh-quit-kill-buffers variable, and its support in
;; 	 mh-quit.
;; 
;; Tue May 29 11:35:38 1990 by Alan Stebbens (aks at somewhere.ucsb.edu)
;; 	 Replaced mh-quit with version which invokes mh-quit-hook
;; 	 _before_ restoring the previous window configuration.
;; 
;; Fri May 25 09:48:15 1990 by Alan Stebbens (aks at somewhere.ucsb.edu)
;; 	 Initial version.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; To: info-gnu-emacs@prep.ai.mit.edu
;; Cc: mh-users%ics@uci.edu
;; Subject: mh-folders.el ... Enhancements to mh-e
;; Date: Mon, 04 Jun 90 11:30:48 PDT
;; From: Alan Stebbens <aks@ucsb.edu> <aks@somewhere>
;;
;; I use lots of recursive folders, and have developed the following elisp
;; code to make it easy to use them.   To use this file, in your .emacs
;; file do (or the equivalent:
;;
;; 	(require mh-folders)
;;
;; The first time you use this package, you get is a file in your home
;; directory, called ".folders", in which is kept a list of folders.  Once
;; this file is created, reading it is much faster than invoking the MH
;; command: "folders".  When you create a new folder (usually within Emacs
;; MH), it is automatically added to the list when you quit any MH folder
;; buffer.
;;
;; If you create an MH folder outside of Emacs, and a .folders file already
;; exists, then Emacs MH won't know about it, unless you invoke 
;;
;; 	M-x mh-remake-folder-list
;;
;; Also, as an added bonus, since a change was required to implement
;; folders support correctly, mh-quit now supports killing recursive folder
;; and show buffers, either by configuration or by prefix argument.
;; Also, mh-quit restored the previous window configuration **before**
;; running mh-quit-hook, which, IMHO, was useless.  So, this version does
;; it in the opposite, and more useful order.
;;
;; I've also included a cookie to update the mode line when an "inc" is
;; done.  I got this from the net some time ago, and apologice to the
;; original author for not including his/her name.  If he wants to step
;; forward, please do so, and I'll add it to this file also.  The function
;; in question is mh-inc-folder-init.
;;
;; It would be nice, at some point, if this code were to make it into the
;; mainline distribution.
;;
;; Enjoy
;;
;; Alan Stebbens        <aks@hub.ucsb.edu>             (805) 961-3221
;;      Center for Computational Sciences and Engineering (CCSE)
;;           University of California, Santa Barbara (UCSB)
;;            3111 Engineering I, Santa Barbara, CA 93106
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(require 'mh-e)				; be sure mh-e is loaded first

;; Variables

(defvar mh-quit-kill-buffers t "\
*This variable controls whether or not the show and folder buffers are
killed after quitting, via \\[mh-quit], an MH folder.  If nil, then no
buffers are killed; if t, only the show buffer is killed; if non-nil and
non-t, then kill both the show and folder buffers.  All buffer kills are
done after invoking any functions in the mh-quit-hook list.")

(defvar mh-folder-list-filename ".folders"
  "*If non-nil, this file will be used to save and restore the folder name
list.  Use \\[mh-remake-folder-list] to reinitialize the folder name list.
If this filename does not contain a leading '~' or '/', then it is relative
to \"mh-user-path\".")

;; Functions

;; mh-quit
;; (This is a replacement, since the *real* mh-quit restores 
;; the window-configuration before running the mh-quit-hook functions).

(defun mh-quit (arg) "\
Restore the previous window configuration, if one exists.  Finishes by
running mh-quit-hook if it is bound and non-nil.  If invoked with C-u
prefix arg, it temporarily toggles the value of mh-quit-kill-buffers
with nil.  An arg of C-u 1 causes both the current folder and show
buffers to be killed."
  (interactive "P")
  (let* ((flag (cond ((null arg) mh-quit-kill-buffers)
		     ((atom arg) arg)
		     (t (not mh-quit-kill-buffers))))
	 (folder-buffer (get-buffer mh-current-folder))
	 (show-buffer (get-buffer mh-show-buffer)))
    (run-hooks 'mh-quit-hook)
    (if mh-previous-window-config
	(set-window-configuration mh-previous-window-config))
    (if (and (not (null flag))
	     show-buffer)
	(kill-buffer show-buffer))
    (if (and (not (null flag))
	     (not (eq t flag))
	     folder-buffer)
	(kill-buffer folder-buffer))))
	     
;; mh-remake-folder-list

(defun mh-remake-folder-list ()
  "Force the folder name list to be recreated from the output of the MH 
\"folders\" command.  If \"mh-folder-name-list\" is non-nil, the folder name
list will be saved into this file, and restored on subsequent loads of the
MH package."
  (interactive)
  (if (and (not (null mh-folder-list-filename))
	   (file-exists-p mh-folder-list-filename))
      (delete-file (mh-folder-list-filename)))
  (setq mh-folder-list (mh-make-folder-list))
  (if mh-folder-list-filename
      (mh-save-folder-list mh-folder-list)))

;; mh-make-folder-list
;;
;; Since this function already exists in the MH-E package, and we want to
;; "replace" it, without losing its functionality, we need to keep the 
;; original function def somewhere.  This will break, however, if mh-e is
;; loaded _after_ this package (which shouldn't happen with the require mh-e
;; above).

(if (not (fboundp 'mh-make-folder-list-orig))
    (fset 'mh-make-folder-list-orig (symbol-function 'mh-make-folder-list)))

(defun mh-make-folder-list ()
  "Return a list of the user's folders.  If mh-folder-list-filename is non-nil, 
then read this file for the stored list of folder names; this is much faster 
than executing the MH \"folders\" command.  However, if the prefix ARG is
given, then force the invocation of the \"folders\" command to rebuild the
folder list."
  (interactive)
  (if (or (null mh-folder-list-filename)
	  (not (file-readable-p mh-folder-list-filename)))
      (save-excursion
	(get-buffer-create " *mh-temp*")
	(let ((list (mh-make-folder-list-orig)))
	  (if mh-folder-list-filename (mh-save-folder-list list))
	  list))
    (mh-read-folder-list)))

(defun mh-save-folder-list (list)
  "Write LIST to the file named by \"mh-folder-list-filename\"."
  (save-window-excursion
    (let* ((file (expand-file-name mh-folder-list-filename mh-user-path))
	   (buf (set-buffer (create-file-buffer file))))
      (set-visited-file-name file)
      (erase-buffer)
      (prin1 list buf)
      (princ "\n" buf)
      (save-buffer 0)
      (kill-buffer buf))))

(defun mh-read-folder-list ()
  "Read the current folder list from the file named by \"mh-folder-list-filename\".
The list is the return value."
  (let (list
	(file (expand-file-name mh-folder-list-filename mh-user-path)))
    (save-window-excursion
      (if (not (file-readable-p file))
	  (error "Can't read %s for list of folder names." file)
	(find-file-read-only file)
	(setq list (read (current-buffer)))
	(kill-buffer (current-buffer))))
    list))

(if (not (memq 'mh-save-folder-list-quit mh-quit-hook))
    (setq mh-quit-hook (append mh-quit-hook (list 'mh-save-folder-list-quit))))

(defun mh-save-folder-list-quit ()
  "Save the folder list before quitting MH"
  (interactive)
  (if mh-folder-list 
      (mh-save-folder-list mh-folder-list))
  (bury-buffer (set-buffer mh-current-folder))
  (if mh-show-buffer
      (bury-buffer (get-buffer mh-show-buffer))))

;; mh-inc-folder-init

;; Function to update the modeline when we "inc"

(defun mh-inc-folder-init ()
  "Initialize the current folder display after we run \"inc\"."

  ;; Remove the 'Mail' signal from the modeline
  (if (and (boundp 'display-time-process)
	   (eq 'run (process-status display-time-process)))
      (let ((real-match-data (match-data)))
	(unwind-protect
	    (if (string-match " Mail" display-time-string)
		(setq display-time-string
		      (concat 
		       (substring display-time-string 0 (match-beginning 0))
		       (substring display-time-string (match-end 0)))))
	  (store-match-data real-match-data))
	(set-buffer-modified-p (buffer-modified-p)))))

(if (not (memq 'mh-inc-folder-init mh-inc-folder-hook))
    (setq mh-inc-folder-hook (append mh-inc-folder-hook '(mh-inc-folder-init))))

;; Announce that we're loaded

(provide 'mh-folders)
